/*
 * version.c - 18 December 2012
 *
 * This program has been placed under the GPL.  Any bugfixes or enhancements
 * will be greatly appreciated :)
 */

/* The version number :) */
static const int MAJOR_VERSION = 1;
static const int MINOR_VERSION = 14;
